class Actions:

    def __init__(self):
        pass

    Install = 'Install'
    Upgrade = 'Upgrade'
    Backup = 'Backup'
    Restore = 'Restore'


class Automation:

    def __init__(self):
        pass

    Normal = 'normal'
    Save = 'save'
    Load = 'load'
